@@ -150,6 +150,42 @@ public class JdkClassReader {
         return classModel;
     }
 
+    /**
+     * Returns the access flags of this class as stored in the class file.
+     *
+     * @return The access flags of this class as stored in the class file.
+     */
+    public int getAccess() {
+        return classModel.flags().flagsMask();
+    }
+
+    /**
+     * Returns the internal name of this class.
+     *
+     * @return The internal name of this class.
+     */
+    public String getClassName() {
+        return classModel.thisClass().asInternalName();
+    }
+
+    /**
+     * Returns the internal super class name of this class or {@code null} for {@link Object}.
+     *
+     * @return The internal super class name of this class or {@code null} for {@link Object}.
+     */
+    public String getSuperName() {
+        return classModel.superclass().map(ClassEntry::asInternalName).orElse(null);
+    }
+
+    /**
+     * Returns the internal interface names of this class. Maybe {@code null}.
+     *
+     * @return The internal interface names of this class.
+     */
+    public String[] getInterfaces() {
+        return classModel.interfaces().stream().map(ClassEntry::asInternalName).toArray(String[]::new);
+    }
+
     /**
      * Accepts a class visitor for the represented class file.
      *
@@ -159,14 +195,14 @@ public class JdkClassReader {
     public void accept(ClassVisitor classVisitor, int flags) {
         Map<Label, org.objectweb.asm.Label> labels = new HashMap<>();
         classVisitor.visit(classModel.minorVersion() << 16 | classModel.majorVersion(),
-                classModel.flags().flagsMask()
+                getAccess()
                         | (classModel.findAttribute(Attributes.deprecated()).isPresent() ? Opcodes.ACC_DEPRECATED : 0)
-                        | (classModel.findAttribute(Attributes.synthetic()).isPresent() ? Opcodes.ACC_SYNTHETIC: 0)
+                        | (classModel.findAttribute(Attributes.synthetic()).isPresent() ? Opcodes.ACC_SYNTHETIC : 0)
                         | (classModel.findAttribute(Attributes.record()).isPresent() ? Opcodes.ACC_RECORD : 0),
-                classModel.thisClass().asInternalName(),
+                getClassName(),
                 classModel.findAttribute(Attributes.signature()).map(signature -> signature.signature().stringValue()).orElse(null),
-                classModel.superclass().map(ClassEntry::asInternalName).orElse(null),
-                classModel.interfaces().stream().map(ClassEntry::asInternalName).toArray(String[]::new));
+                getSuperName(),
+                getInterfaces());
         if ((flags & ClassReader.SKIP_DEBUG) == 0) {
             String sourceFile = classModel.findAttribute(Attributes.sourceFile())
                     .map(attribute -> attribute.sourceFile().stringValue())
@@ -658,11 +694,11 @@ public class JdkClassReader {
     }
 
     private void appendCodeAnnotations(List<TypeAnnotation> typeAnnotations,
-                                              boolean visible,
-                                              MethodVisitor methodVisitor,
-                                              Map<Label, org.objectweb.asm.Label> labels,
-                                              List<Map.Entry<TypeAnnotation, Boolean>> localVariableAnnotations,
-                                              Map<org.objectweb.asm.Label, List<Map.Entry<TypeAnnotation, Boolean>>> offsetTypeAnnotations) {
+                                       boolean visible,
+                                       MethodVisitor methodVisitor,
+                                       Map<Label, org.objectweb.asm.Label> labels,
+                                       List<Map.Entry<TypeAnnotation, Boolean>> localVariableAnnotations,
+                                       Map<org.objectweb.asm.Label, List<Map.Entry<TypeAnnotation, Boolean>>> offsetTypeAnnotations) {
         typeAnnotations.forEach(typeAnnotation -> {
             switch (typeAnnotation.targetInfo()) {
                 case TypeAnnotation.LocalVarTarget ignored -> localVariableAnnotations.add(Map.entry(typeAnnotation, visible));
