@@ -640,13 +640,11 @@ public class JdkClassWriter extends ClassVisitor {
                 Type type = Type.getMethodType(descriptor);
                 for (Type argumentType : type.getArgumentTypes()) {
                     locals.add(switch (argumentType.getSort()) {
-                        case Type.BOOLEAN, Type.BYTE, Type.SHORT, Type.CHAR, Type.INT ->
-                                StackMapFrameInfo.SimpleVerificationTypeInfo.INTEGER;
+                        case Type.BOOLEAN, Type.BYTE, Type.SHORT, Type.CHAR, Type.INT -> StackMapFrameInfo.SimpleVerificationTypeInfo.INTEGER;
                         case Type.LONG -> StackMapFrameInfo.SimpleVerificationTypeInfo.LONG;
                         case Type.FLOAT -> StackMapFrameInfo.SimpleVerificationTypeInfo.FLOAT;
                         case Type.DOUBLE -> StackMapFrameInfo.SimpleVerificationTypeInfo.DOUBLE;
-                        default ->
-                                StackMapFrameInfo.ObjectVerificationTypeInfo.of(ClassDesc.ofDescriptor(argumentType.getDescriptor()));
+                        default -> StackMapFrameInfo.ObjectVerificationTypeInfo.of(ClassDesc.ofDescriptor(argumentType.getDescriptor()));
                     });
                 }
             }
@@ -929,8 +927,7 @@ public class JdkClassWriter extends ClassVisitor {
                 case Opcodes.FSTORE -> codeBuilder -> codeBuilder.fstore(varIndex);
                 case Opcodes.DSTORE -> codeBuilder -> codeBuilder.dstore(varIndex);
                 case Opcodes.ASTORE -> codeBuilder -> codeBuilder.astore(varIndex);
-                case Opcodes.RET ->
-                        codeBuilder -> codeBuilder.with(DiscontinuedInstruction.RetInstruction.of(varIndex));
+                case Opcodes.RET -> codeBuilder -> codeBuilder.with(DiscontinuedInstruction.RetInstruction.of(varIndex));
                 default -> throw new IllegalArgumentException("Unexpected opcode: " + opcode);
             };
             addInstruction(codeConsumer);
@@ -973,42 +970,24 @@ public class JdkClassWriter extends ClassVisitor {
         @Override
         public void visitJumpInsn(int opcode, Label label) {
             Consumer<CodeBuilder> codeConsumer = switch (opcode) {
-                case Opcodes.IFEQ ->
-                        codeBuilder -> codeBuilder.ifeq(labels.computeIfAbsent(label, _ -> codeBuilder.newLabel()));
-                case Opcodes.IFNE ->
-                        codeBuilder -> codeBuilder.ifne(labels.computeIfAbsent(label, _ -> codeBuilder.newLabel()));
-                case Opcodes.IFLT ->
-                        codeBuilder -> codeBuilder.iflt(labels.computeIfAbsent(label, _ -> codeBuilder.newLabel()));
-                case Opcodes.IFGE ->
-                        codeBuilder -> codeBuilder.ifge(labels.computeIfAbsent(label, _ -> codeBuilder.newLabel()));
-                case Opcodes.IFGT ->
-                        codeBuilder -> codeBuilder.ifgt(labels.computeIfAbsent(label, _ -> codeBuilder.newLabel()));
-                case Opcodes.IFLE ->
-                        codeBuilder -> codeBuilder.ifle(labels.computeIfAbsent(label, _ -> codeBuilder.newLabel()));
-                case Opcodes.IF_ICMPEQ ->
-                        codeBuilder -> codeBuilder.if_icmpeq(labels.computeIfAbsent(label, _ -> codeBuilder.newLabel()));
-                case Opcodes.IF_ICMPNE ->
-                        codeBuilder -> codeBuilder.if_icmpne(labels.computeIfAbsent(label, _ -> codeBuilder.newLabel()));
-                case Opcodes.IF_ICMPLT ->
-                        codeBuilder -> codeBuilder.if_icmplt(labels.computeIfAbsent(label, _ -> codeBuilder.newLabel()));
-                case Opcodes.IF_ICMPGE ->
-                        codeBuilder -> codeBuilder.if_icmpge(labels.computeIfAbsent(label, _ -> codeBuilder.newLabel()));
-                case Opcodes.IF_ICMPGT ->
-                        codeBuilder -> codeBuilder.if_icmpgt(labels.computeIfAbsent(label, _ -> codeBuilder.newLabel()));
-                case Opcodes.IF_ICMPLE ->
-                        codeBuilder -> codeBuilder.if_icmple(labels.computeIfAbsent(label, _ -> codeBuilder.newLabel()));
-                case Opcodes.IF_ACMPEQ ->
-                        codeBuilder -> codeBuilder.if_acmpeq(labels.computeIfAbsent(label, _ -> codeBuilder.newLabel()));
-                case Opcodes.IF_ACMPNE ->
-                        codeBuilder -> codeBuilder.if_acmpne(labels.computeIfAbsent(label, _ -> codeBuilder.newLabel()));
-                case Opcodes.GOTO ->
-                        codeBuilder -> codeBuilder.goto_(labels.computeIfAbsent(label, _ -> codeBuilder.newLabel()));
-                case Opcodes.IFNULL ->
-                        codeBuilder -> codeBuilder.ifnull(labels.computeIfAbsent(label, _ -> codeBuilder.newLabel()));
-                case Opcodes.IFNONNULL ->
-                        codeBuilder -> codeBuilder.ifnonnull(labels.computeIfAbsent(label, _ -> codeBuilder.newLabel()));
-                case Opcodes.JSR ->
-                        codeBuilder -> codeBuilder.with(DiscontinuedInstruction.JsrInstruction.of(labels.computeIfAbsent(label, _ -> codeBuilder.newLabel())));
+                case Opcodes.IFEQ -> codeBuilder -> codeBuilder.ifeq(labels.computeIfAbsent(label, _ -> codeBuilder.newLabel()));
+                case Opcodes.IFNE -> codeBuilder -> codeBuilder.ifne(labels.computeIfAbsent(label, _ -> codeBuilder.newLabel()));
+                case Opcodes.IFLT -> codeBuilder -> codeBuilder.iflt(labels.computeIfAbsent(label, _ -> codeBuilder.newLabel()));
+                case Opcodes.IFGE -> codeBuilder -> codeBuilder.ifge(labels.computeIfAbsent(label, _ -> codeBuilder.newLabel()));
+                case Opcodes.IFGT -> codeBuilder -> codeBuilder.ifgt(labels.computeIfAbsent(label, _ -> codeBuilder.newLabel()));
+                case Opcodes.IFLE -> codeBuilder -> codeBuilder.ifle(labels.computeIfAbsent(label, _ -> codeBuilder.newLabel()));
+                case Opcodes.IF_ICMPEQ -> codeBuilder -> codeBuilder.if_icmpeq(labels.computeIfAbsent(label, _ -> codeBuilder.newLabel()));
+                case Opcodes.IF_ICMPNE -> codeBuilder -> codeBuilder.if_icmpne(labels.computeIfAbsent(label, _ -> codeBuilder.newLabel()));
+                case Opcodes.IF_ICMPLT -> codeBuilder -> codeBuilder.if_icmplt(labels.computeIfAbsent(label, _ -> codeBuilder.newLabel()));
+                case Opcodes.IF_ICMPGE -> codeBuilder -> codeBuilder.if_icmpge(labels.computeIfAbsent(label, _ -> codeBuilder.newLabel()));
+                case Opcodes.IF_ICMPGT -> codeBuilder -> codeBuilder.if_icmpgt(labels.computeIfAbsent(label, _ -> codeBuilder.newLabel()));
+                case Opcodes.IF_ICMPLE -> codeBuilder -> codeBuilder.if_icmple(labels.computeIfAbsent(label, _ -> codeBuilder.newLabel()));
+                case Opcodes.IF_ACMPEQ -> codeBuilder -> codeBuilder.if_acmpeq(labels.computeIfAbsent(label, _ -> codeBuilder.newLabel()));
+                case Opcodes.IF_ACMPNE -> codeBuilder -> codeBuilder.if_acmpne(labels.computeIfAbsent(label, _ -> codeBuilder.newLabel()));
+                case Opcodes.GOTO -> codeBuilder -> codeBuilder.goto_(labels.computeIfAbsent(label, _ -> codeBuilder.newLabel()));
+                case Opcodes.IFNULL -> codeBuilder -> codeBuilder.ifnull(labels.computeIfAbsent(label, _ -> codeBuilder.newLabel()));
+                case Opcodes.IFNONNULL -> codeBuilder -> codeBuilder.ifnonnull(labels.computeIfAbsent(label, _ -> codeBuilder.newLabel()));
+                case Opcodes.JSR -> codeBuilder -> codeBuilder.with(DiscontinuedInstruction.JsrInstruction.of(labels.computeIfAbsent(label, _ -> codeBuilder.newLabel())));
                 default -> throw new IllegalArgumentException("Unexpected opcode: " + opcode);
             };
             addInstruction(codeConsumer);
@@ -1193,20 +1172,7 @@ public class JdkClassWriter extends ClassVisitor {
 
     @Override
     public void visitEnd() {
-        ClassFile classFile;
-        if ((flags & ClassWriter.COMPUTE_FRAMES) == 0) {
-            classFile = ClassFile.of(ClassFile.DeadCodeOption.KEEP_DEAD_CODE, ClassFile.StackMapsOption.DROP_STACK_MAPS);
-        } else {
-            classFile = ClassFile.of(ClassFile.DeadCodeOption.PATCH_DEAD_CODE, ClassFile.StackMapsOption.STACK_MAPS_WHEN_REQUIRED, ClassFile.ClassHierarchyResolverOption.of(classDesc -> {
-                if (!classDesc.isClassOrInterface()) {
-                    return null;
-                } else if (classDesc.equals(ConstantDescs.CD_Object)) {
-                    return ClassHierarchyResolver.ClassHierarchyInfo.ofClass(null);
-                }
-                String superClass = getSuperClass(classDesc.displayName().replace('.', '/'));
-                return superClass == null ? ClassHierarchyResolver.ClassHierarchyInfo.ofInterface() : ClassHierarchyResolver.ClassHierarchyInfo.ofClass(ClassDesc.ofInternalName(superClass));
-            }));
-        }
+        ClassFile classFile = getClassFile(flags);
         if (classModel == null) {
             bytes = classFile.build(thisClass, classBuilder -> classConsumers.forEach(classConsumer -> classConsumer.accept(classBuilder)));
         } else {
@@ -1215,6 +1181,28 @@ public class JdkClassWriter extends ClassVisitor {
         }
     }
 
+    /**
+     * Returns an appropriate {@code ClassFile} instance. Can be overridden to return custom instances.
+     *
+     * @param flags {@link ClassWriter} flags.
+     * @return An instance of {@code ClassFile} configured according to the given flags.
+     */
+    protected ClassFile getClassFile(int flags) {
+        if ((flags & ClassWriter.COMPUTE_FRAMES) == 0) {
+            return ClassFile.of(ClassFile.DeadCodeOption.KEEP_DEAD_CODE, ClassFile.StackMapsOption.DROP_STACK_MAPS);
+        } else {
+            return ClassFile.of(ClassFile.DeadCodeOption.PATCH_DEAD_CODE, ClassFile.StackMapsOption.STACK_MAPS_WHEN_REQUIRED, ClassFile.ClassHierarchyResolverOption.of(classDesc -> {
+                if (!classDesc.isClassOrInterface()) {
+                    return null;
+                } else if (classDesc.equals(ConstantDescs.CD_Object)) {
+                    return ClassHierarchyResolver.ClassHierarchyInfo.ofClass(null);
+                }
+                String descriptor = classDesc.descriptorString(), superClass = getSuperClass(descriptor.substring(1, descriptor.length()  - 1));
+                return superClass == null ? ClassHierarchyResolver.ClassHierarchyInfo.ofInterface() : ClassHierarchyResolver.ClassHierarchyInfo.ofClass(ClassDesc.ofInternalName(superClass));
+            }));
+        }
+    }
+
     static ConstantDesc toConstantDesc(Object asm) {
         return switch (asm) {
             case Integer value -> value;
